@@ -8,18 +8,18 @@
 // <int>.<int>.<int>-<any text>.
 // or else upgrading old project file will not work
 // reliably.
-#define VERSION "3.2.2-Bonn"
+#define VERSION "3.4.4-Madeira"
 
 //used in vim src/core/qgis.cpp
 //The way below should work but it resolves to a number like 0110 which the compiler treats as octal I think
 //because debuggin it out shows the decimal number 72 which results in incorrect version status.
 //As a short term fix I (Tim) am defining the version in top level cmake. It would be good to
 //reinstate this more generic approach below at some point though
-//#define VERSION_INT 322
-#define VERSION_INT 30202
-#define ABISYM(x) x ## 30202
+//#define VERSION_INT 344
+#define VERSION_INT 30404
+#define ABISYM(x) x ## 30404
 //used in main.cpp and anywhere else where the release name is needed
-#define RELEASE_NAME "Bonn"
+#define RELEASE_NAME "Madeira"
 
 #define QGIS_PLUGIN_SUBDIR "lib/qgis/plugins"
 #define QGIS_DATA_SUBDIR "share/qgis"
@@ -27,11 +27,11 @@
 #define QGIS_LIB_SUBDIR "lib"
 #define QGIS_QML_SUBDIR "qml"
 #define CMAKE_INSTALL_PREFIX "/usr"
-#define CMAKE_SOURCE_DIR "/build/qgis/src/qgis-3.2.2"
+#define CMAKE_SOURCE_DIR "/build/qgis/src/qgis-3.4.4"
 
 #define QGIS_SERVER_MODULE_SUBDIR "lib/qgis/server"
 
-#define QSCINTILLA_VERSION_STR "2.10.7"
+#define QSCINTILLA_VERSION_STR "2.11.1"
 
 #if defined( __APPLE__ )
 //used by Mac to find system or bundle resources relative to amount of bundling
@@ -57,6 +57,8 @@
 
 #define HAVE_SERVER_PYTHON_PLUGINS
 
+/* #undef HAVE_OPENCL */
+
 /* #undef ENABLE_MODELTEST */
 
 #define HAVE_3D
