@@ -24,6 +24,7 @@
 #include "qgsproperty.h"
 #include "qgscoordinatereferencesystem.h"
 #include "qgsfeaturesource.h"
+#include "qgsprocessingutils.h"
 #include <QMap>
 #include <limits>
 
@@ -153,6 +154,21 @@ class CORE_EXPORT QgsProcessingOutputLayerDefinition
      */
     QVariantMap createOptions;
 
+    /**
+     * Saves this output layer definition to a QVariantMap, wrapped in a QVariant.
+     * You can use QgsXmlUtils::writeVariant to save it to an XML document.
+     * \see loadVariant()
+     * \since QGIS 3.2
+     */
+    QVariant toVariant() const;
+
+    /**
+     * Loads this output layer definition from a QVariantMap, wrapped in a QVariant.
+     * You can use QgsXmlUtils::readVariant to load it from an XML document.
+     * \see toVariant()
+     * \since QGIS 3.2
+     */
+    bool loadVariant( const QVariantMap &map );
 
     //! Allows direct construction of QVariants.
     operator QVariant() const
@@ -206,6 +222,8 @@ class CORE_EXPORT QgsProcessingParameterDefinition
       sipType = sipType_QgsProcessingParameterMultipleLayers;
     else if ( sipCpp->type() == QgsProcessingParameterNumber::typeName() )
       sipType = sipType_QgsProcessingParameterNumber;
+    else if ( sipCpp->type() == QgsProcessingParameterDistance::typeName() )
+      sipType = sipType_QgsProcessingParameterDistance;
     else if ( sipCpp->type() == QgsProcessingParameterRange::typeName() )
       sipType = sipType_QgsProcessingParameterRange;
     else if ( sipCpp->type() == QgsProcessingParameterRasterLayer::typeName() )
@@ -247,6 +265,7 @@ class CORE_EXPORT QgsProcessingParameterDefinition
       FlagAdvanced = 1 << 1, //!< Parameter is an advanced parameter which should be hidden from users by default
       FlagHidden = 1 << 2, //!< Parameter is hidden and should not be shown to users
       FlagOptional = 1 << 3, //!< Parameter is optional
+      FlagIsModelOutput = 1 << 4, //!< Destination parameter is final output. The parameter name will be used.
     };
     Q_DECLARE_FLAGS( Flags, Flag )
 
@@ -366,7 +385,7 @@ class CORE_EXPORT QgsProcessingParameterDefinition
 
     /**
      * Returns the parameter's freeform metadata. This is mostly used by parameter widget wrappers
-     * in order to customise their appearance and behavior.
+     * in order to customize their appearance and behavior.
      * \see setMetadata()
      * \note not available in Python bindings.
      */
@@ -374,14 +393,14 @@ class CORE_EXPORT QgsProcessingParameterDefinition
 
     /**
      * Returns the parameter's freeform metadata. This is mostly used by parameter widget wrappers
-     * in order to customise their appearance and behavior.
+     * in order to customize their appearance and behavior.
      * \see setMetadata()
      */
     QVariantMap &metadata() { return mMetadata; }
 
     /**
      * Sets the parameter's freeform \a metadata. This is mostly used by parameter widget wrappers
-     * in order to customise their appearance and behavior.
+     * in order to customize their appearance and behavior.
      * \see metadata()
      */
     void setMetadata( const QVariantMap &metadata ) { mMetadata = metadata; }
@@ -486,7 +505,7 @@ class CORE_EXPORT QgsProcessingParameterDefinition
     //! Parameter flags
     Flags mFlags;
 
-    //! Freeform metadata for parameter. Mostly used by widget wrappers to customise their appearance and behavior.
+    //! Freeform metadata for parameter. Mostly used by widget wrappers to customize their appearance and behavior.
     QVariantMap mMetadata;
 
     //! Pointer to algorithm which owns this parameter
@@ -545,36 +564,90 @@ class CORE_EXPORT QgsProcessingParameters
      */
     static QString parameterAsString( const QgsProcessingParameterDefinition *definition, const QVariantMap &parameters, const QgsProcessingContext &context );
 
+    /**
+     * Evaluates the parameter with matching \a definition and \a value to a static string value.
+     * \since QGIS 3.4
+     */
+    static QString parameterAsString( const QgsProcessingParameterDefinition *definition, const QVariant &value, const QgsProcessingContext &context );
+
     /**
      * Evaluates the parameter with matching \a definition to an expression.
      */
     static QString parameterAsExpression( const QgsProcessingParameterDefinition *definition, const QVariantMap &parameters, const QgsProcessingContext &context );
 
+    /**
+     * Evaluates the parameter with matching \a definitionand \a value to an expression.
+     * \since QGIS 3.4
+     */
+    static QString parameterAsExpression( const QgsProcessingParameterDefinition *definition, const QVariant &value, const QgsProcessingContext &context );
+
     /**
      * Evaluates the parameter with matching \a definition to a static double value.
      */
     static double parameterAsDouble( const QgsProcessingParameterDefinition *definition, const QVariantMap &parameters, const QgsProcessingContext &context );
 
+    /**
+     * Evaluates the parameter with matching \a definition and \a value to a static double value.
+     * \since QGIS 3.4
+     */
+    static double parameterAsDouble( const QgsProcessingParameterDefinition *definition, const QVariant &value, const QgsProcessingContext &context );
+
     /**
      * Evaluates the parameter with matching \a definition to a static integer value.
      */
     static int parameterAsInt( const QgsProcessingParameterDefinition *definition, const QVariantMap &parameters, const QgsProcessingContext &context );
 
+    /**
+     * Evaluates the parameter with matching \a definition and \a value to a static integer value.
+     * \since QGIS 3.4
+     */
+    static int parameterAsInt( const QgsProcessingParameterDefinition *definition, const QVariant &value, const QgsProcessingContext &context );
+
+    /**
+     * Evaluates the parameter with matching \a definition to a list of integer values.
+     * \since QGIS 3.4
+     */
+    static QList<int> parameterAsInts( const QgsProcessingParameterDefinition *definition, const QVariantMap &parameters, const QgsProcessingContext &context );
+
+    /**
+     * Evaluates the parameter with matching \a definition and \a value to a list of integer values.
+     * \since QGIS 3.4
+     */
+    static QList<int> parameterAsInts( const QgsProcessingParameterDefinition *definition, const QVariant &value, const QgsProcessingContext &context );
+
     /**
      * Evaluates the parameter with matching \a definition to a enum value.
      */
     static int parameterAsEnum( const QgsProcessingParameterDefinition *definition, const QVariantMap &parameters, const QgsProcessingContext &context );
 
+    /**
+     * Evaluates the parameter with matching \a definition and \a value to a enum value.
+     * \since QGIS 3.4
+     */
+    static int parameterAsEnum( const QgsProcessingParameterDefinition *definition, const QVariant &value, const QgsProcessingContext &context );
+
     /**
      * Evaluates the parameter with matching \a definition to list of enum values.
      */
     static QList<int> parameterAsEnums( const QgsProcessingParameterDefinition *definition, const QVariantMap &parameters, const QgsProcessingContext &context );
 
+    /**
+     * Evaluates the parameter with matching \a definition and \a value to list of enum values.
+     * \since QGIS 3.4
+     */
+    static QList<int> parameterAsEnums( const QgsProcessingParameterDefinition *definition, const QVariant &value, const QgsProcessingContext &context );
+
     /**
      * Evaluates the parameter with matching \a definition to a static boolean value.
      */
     static bool parameterAsBool( const QgsProcessingParameterDefinition *definition, const QVariantMap &parameters, const QgsProcessingContext &context );
 
+    /**
+     * Evaluates the parameter with matching \a definition and \a value to a static boolean value.
+     * \since QGIS 3.4
+     */
+    static bool parameterAsBool( const QgsProcessingParameterDefinition *definition, const QVariant &value, const QgsProcessingContext &context );
+
     /**
      * Evaluates the parameter with matching \a definition to a feature sink.
      *
@@ -590,7 +663,26 @@ class CORE_EXPORT QgsProcessingParameters
      */
     static QgsFeatureSink *parameterAsSink( const QgsProcessingParameterDefinition *definition, const QVariantMap &parameters,
                                             const QgsFields &fields, QgsWkbTypes::Type geometryType, const QgsCoordinateReferenceSystem &crs,
-                                            QgsProcessingContext &context, QString &destinationIdentifier SIP_OUT ) SIP_FACTORY;
+                                            QgsProcessingContext &context, QString &destinationIdentifier SIP_OUT, QgsFeatureSink::SinkFlags sinkFlags = nullptr ) SIP_FACTORY;
+
+    /**
+     * Evaluates the parameter with matching \a definition and \a value to a feature sink.
+     *
+     * The \a fields, \a geometryType and \a crs parameters dictate the properties
+     * of the resulting feature sink.
+     *
+     * Sinks will either be taken from \a context's active project, or created from external
+     * providers and stored temporarily in the \a context. The \a destinationIdentifier
+     * argument will be set to a string which can be used to retrieve the layer corresponding
+     * to the sink, e.g. via calling QgsProcessingUtils::mapLayerFromString().
+     *
+     * This function creates a new object and the caller takes responsibility for deleting the returned object.
+     *
+     * \since QGIS 3.4
+     */
+    static QgsFeatureSink *parameterAsSink( const QgsProcessingParameterDefinition *definition, const QVariant &value,
+                                            const QgsFields &fields, QgsWkbTypes::Type geometryType, const QgsCoordinateReferenceSystem &crs,
+                                            QgsProcessingContext &context, QString &destinationIdentifier SIP_OUT, QgsFeatureSink::SinkFlags sinkFlags = nullptr ) SIP_FACTORY;
 
     /**
      * Evaluates the parameter with matching \a definition to a feature source.
@@ -602,6 +694,18 @@ class CORE_EXPORT QgsProcessingParameters
      */
     static QgsProcessingFeatureSource *parameterAsSource( const QgsProcessingParameterDefinition *definition, const QVariantMap &parameters, QgsProcessingContext &context ) SIP_FACTORY;
 
+    /**
+     * Evaluates the parameter with matching \a definition and \a value to a feature source.
+     *
+     * Sources will either be taken from \a context's active project, or loaded from external
+     * sources and stored temporarily in the \a context.
+     *
+     * This function creates a new object and the caller takes responsibility for deleting the returned object.
+     *
+     * \since QGIS 3.4
+     */
+    static QgsProcessingFeatureSource *parameterAsSource( const QgsProcessingParameterDefinition *definition, const QVariant &value, QgsProcessingContext &context ) SIP_FACTORY;
+
     /**
      * Evaluates the parameter with matching \a definition to a source vector layer file path of compatible format.
      *
@@ -626,6 +730,17 @@ class CORE_EXPORT QgsProcessingParameters
      */
     static QgsMapLayer *parameterAsLayer( const QgsProcessingParameterDefinition *definition, const QVariantMap &parameters, QgsProcessingContext &context );
 
+    /**
+     * Evaluates the parameter with matching \a definition and \a value to a map layer.
+     *
+     * Layers will either be taken from \a context's active project, or loaded from external
+     * sources and stored temporarily in the \a context. In either case, callers do not
+     * need to handle deletion of the returned layer.
+     *
+     * \since QGIS 3.4
+     */
+    static QgsMapLayer *parameterAsLayer( const QgsProcessingParameterDefinition *definition, const QVariant &value, QgsProcessingContext &context );
+
     /**
      * Evaluates the parameter with matching \a definition to a raster layer.
      *
@@ -635,16 +750,39 @@ class CORE_EXPORT QgsProcessingParameters
      */
     static QgsRasterLayer *parameterAsRasterLayer( const QgsProcessingParameterDefinition *definition, const QVariantMap &parameters, QgsProcessingContext &context );
 
+    /**
+     * Evaluates the parameter with matching \a definition and \a value to a raster layer.
+     *
+     * Layers will either be taken from \a context's active project, or loaded from external
+     * sources and stored temporarily in the \a context. In either case, callers do not
+     * need to handle deletion of the returned layer.
+     *
+     * \since QGIS 3.4
+     */
+    static QgsRasterLayer *parameterAsRasterLayer( const QgsProcessingParameterDefinition *definition, const QVariant &value, QgsProcessingContext &context );
+
     /**
      * Evaluates the parameter with matching \a definition to a output layer destination.
      */
     static QString parameterAsOutputLayer( const QgsProcessingParameterDefinition *definition, const QVariantMap &parameters, QgsProcessingContext &context );
 
+    /**
+     * Evaluates the parameter with matching \a definition and \a value to a output layer destination.
+     * \since QGIS 3.4
+     */
+    static QString parameterAsOutputLayer( const QgsProcessingParameterDefinition *definition, const QVariant &value, QgsProcessingContext &context );
+
     /**
      * Evaluates the parameter with matching \a definition to a file based output destination.
      */
     static QString parameterAsFileOutput( const QgsProcessingParameterDefinition *definition, const QVariantMap &parameters, QgsProcessingContext &context );
 
+    /**
+     * Evaluates the parameter with matching \a definition and \a value to a file based output destination.
+     * \since QGIS 3.4
+     */
+    static QString parameterAsFileOutput( const QgsProcessingParameterDefinition *definition, const QVariant &value, QgsProcessingContext &context );
+
     /**
      * Evaluates the parameter with matching \a definition to a vector layer.
      *
@@ -654,11 +792,28 @@ class CORE_EXPORT QgsProcessingParameters
      */
     static QgsVectorLayer *parameterAsVectorLayer( const QgsProcessingParameterDefinition *definition, const QVariantMap &parameters, QgsProcessingContext &context );
 
+    /**
+     * Evaluates the parameter with matching \a definition and \a value to a vector layer.
+     *
+     * Layers will either be taken from \a context's active project, or loaded from external
+     * sources and stored temporarily in the \a context. In either case, callers do not
+     * need to handle deletion of the returned layer.
+     *
+     * \since QGIS 3.4
+     */
+    static QgsVectorLayer *parameterAsVectorLayer( const QgsProcessingParameterDefinition *definition, const QVariant &value, QgsProcessingContext &context );
+
     /**
      * Evaluates the parameter with matching \a definition to a coordinate reference system.
      */
     static QgsCoordinateReferenceSystem parameterAsCrs( const QgsProcessingParameterDefinition *definition, const QVariantMap &parameters, QgsProcessingContext &context );
 
+    /**
+     * Evaluates the parameter with matching \a definition and \a value to a coordinate reference system.
+     * \since QGIS 3.4
+     */
+    static QgsCoordinateReferenceSystem parameterAsCrs( const QgsProcessingParameterDefinition *definition, const QVariant &value, QgsProcessingContext &context );
+
     /**
      * Evaluates the parameter with matching \a definition to a rectangular extent.
      *
@@ -671,6 +826,20 @@ class CORE_EXPORT QgsProcessingParameters
     static QgsRectangle parameterAsExtent( const QgsProcessingParameterDefinition *definition, const QVariantMap &parameters, QgsProcessingContext &context,
                                            const QgsCoordinateReferenceSystem &crs = QgsCoordinateReferenceSystem() );
 
+    /**
+     * Evaluates the parameter with matching \a definition and \a value to a rectangular extent.
+     *
+     * If \a crs is set, and the original coordinate reference system of the parameter can be determined, then the extent will be automatically
+     * reprojected so that it is in the specified \a crs. In this case the extent of the reproject rectangle will be returned.
+     *
+     * \see parameterAsExtentGeometry()
+     * \see parameterAsExtentCrs()
+     *
+     * \since QGIS 3.4
+     */
+    static QgsRectangle parameterAsExtent( const QgsProcessingParameterDefinition *definition, const QVariant &value, QgsProcessingContext &context,
+                                           const QgsCoordinateReferenceSystem &crs = QgsCoordinateReferenceSystem() );
+
     /**
      * Evaluates the parameter with matching \a definition to a rectangular extent, and returns a geometry covering this extent.
      *
@@ -702,6 +871,17 @@ class CORE_EXPORT QgsProcessingParameters
     static QgsPointXY parameterAsPoint( const QgsProcessingParameterDefinition *definition, const QVariantMap &parameters, QgsProcessingContext &context,
                                         const QgsCoordinateReferenceSystem &crs = QgsCoordinateReferenceSystem() );
 
+    /**
+     * Evaluates the parameter with matching \a definition and \a value to a point.
+     *
+     * If \a crs is set then the point will be automatically reprojected so that it is in the specified \a crs.
+     *
+     * \see parameterAsPointCrs()
+     * \since QGIS 3.4
+     */
+    static QgsPointXY parameterAsPoint( const QgsProcessingParameterDefinition *definition, const QVariant &value, QgsProcessingContext &context,
+                                        const QgsCoordinateReferenceSystem &crs = QgsCoordinateReferenceSystem() );
+
     /**
      * Returns the coordinate reference system associated with an point parameter value.
      *
@@ -714,27 +894,58 @@ class CORE_EXPORT QgsProcessingParameters
      */
     static QString parameterAsFile( const QgsProcessingParameterDefinition *definition, const QVariantMap &parameters, QgsProcessingContext &context );
 
+    /**
+     * Evaluates the parameter with matching \a definition and \a value to a file/folder name.
+     * \since QGIS 3.4
+     */
+    static QString parameterAsFile( const QgsProcessingParameterDefinition *definition, const QVariant &value, QgsProcessingContext &context );
+
     /**
      * Evaluates the parameter with matching \a definition to a matrix/table of values.
      * Tables are collapsed to a 1 dimensional list.
      */
     static QVariantList parameterAsMatrix( const QgsProcessingParameterDefinition *definition, const QVariantMap &parameters, QgsProcessingContext &context );
 
+    /**
+     * Evaluates the parameter with matching \a definition and \a value to a matrix/table of values.
+     * Tables are collapsed to a 1 dimensional list.
+     * \since QGIS 3.4
+     */
+    static QVariantList parameterAsMatrix( const QgsProcessingParameterDefinition *definition, const QVariant &value, QgsProcessingContext &context );
+
     /**
      * Evaluates the parameter with matching \a definition to a list of map layers.
      */
     static QList< QgsMapLayer *> parameterAsLayerList( const QgsProcessingParameterDefinition *definition, const QVariantMap &parameters, QgsProcessingContext &context );
 
+    /**
+     * Evaluates the parameter with matching \a definition and \a value to a list of map layers.
+     * \since QGIS 3.4
+     */
+    static QList< QgsMapLayer *> parameterAsLayerList( const QgsProcessingParameterDefinition *definition, const QVariant &value, QgsProcessingContext &context );
+
     /**
      * Evaluates the parameter with matching \a definition to a range of values.
      */
     static QList<double> parameterAsRange( const QgsProcessingParameterDefinition *definition, const QVariantMap &parameters, QgsProcessingContext &context );
 
+    /**
+     * Evaluates the parameter with matching \a definition and \a value to a range of values.
+     * \since QGIS 3.4
+     */
+    static QList<double> parameterAsRange( const QgsProcessingParameterDefinition *definition, const QVariant &value, QgsProcessingContext &context );
+
     /**
      * Evaluates the parameter with matching \a definition to a list of fields.
      */
     static QStringList parameterAsFields( const QgsProcessingParameterDefinition *definition, const QVariantMap &parameters, QgsProcessingContext &context );
 
+    /**
+     * Evaluates the parameter with matching \a definition and \a value to a list of fields.
+     * \since QGIS 3.4
+     */
+    static QStringList parameterAsFields( const QgsProcessingParameterDefinition *definition, const QVariant &value, QgsProcessingContext &context );
+
     /**
      * Creates a new QgsProcessingParameterDefinition using the configuration from a
      * supplied variant \a map.
@@ -994,7 +1205,7 @@ class CORE_EXPORT QgsProcessingParameterFile : public QgsProcessingParameterDefi
 /**
  * \class QgsProcessingParameterMatrix
  * \ingroup core
- * An table (matrix) parameter for processing algorithms.
+ * A table (matrix) parameter for processing algorithms.
   * \since QGIS 3.0
  */
 class CORE_EXPORT QgsProcessingParameterMatrix : public QgsProcessingParameterDefinition
@@ -1034,7 +1245,7 @@ class CORE_EXPORT QgsProcessingParameterMatrix : public QgsProcessingParameterDe
      * Returns the fixed number of rows in the table. This parameter only has an
      * effect if hasFixedNumberRows() is true.
      * \see setNumberRows()
-     * \see setFixedNumberRows()
+     * \see setHasFixedNumberRows()
      */
     int numberRows() const;
 
@@ -1042,7 +1253,7 @@ class CORE_EXPORT QgsProcessingParameterMatrix : public QgsProcessingParameterDe
      * Sets the fixed number of \a rows in the table. This parameter only has an
      * effect if hasFixedNumberRows() is true.
      * \see numberRows()
-     * \see setFixedNumberRows()
+     * \see setHasFixedNumberRows()
      */
     void setNumberRows( int rows );
 
@@ -1148,7 +1359,19 @@ class CORE_EXPORT QgsProcessingParameterMultipleLayers : public QgsProcessingPar
  * \class QgsProcessingParameterNumber
  * \ingroup core
  * A numeric parameter for processing algorithms.
-  * \since QGIS 3.0
+ *
+ * For numeric parameters with a dataType() of Double, the number of decimals places
+ * shown in the parameter's widget can be specified by setting the parameter's metadata. For example:
+ *
+ * * \code{.py}
+ *   param = QgsProcessingParameterNumber( 'VAL', 'Threshold', type=QgsProcessingParameter.Double)
+ *   # only show two decimal places in parameter's widgets, not 6:
+ *   param.setMetadata( {'widget_wrapper':
+ *     { 'decimals': 2 }
+ *   })
+ * \endcode
+ *
+ * \since QGIS 3.0
  */
 class CORE_EXPORT QgsProcessingParameterNumber : public QgsProcessingParameterDefinition
 {
@@ -1168,8 +1391,8 @@ class CORE_EXPORT QgsProcessingParameterNumber : public QgsProcessingParameterDe
                                            Type type = Integer,
                                            const QVariant &defaultValue = QVariant(),
                                            bool optional = false,
-                                           double minValue = -DBL_MAX + 1,
-                                           double maxValue = DBL_MAX
+                                           double minValue = std::numeric_limits<double>::lowest() + 1,
+                                           double maxValue = std::numeric_limits<double>::max()
                                          );
 
     /**
@@ -1228,11 +1451,92 @@ class CORE_EXPORT QgsProcessingParameterNumber : public QgsProcessingParameterDe
 
   private:
 
-    double mMin = -DBL_MAX + 1;
-    double mMax = DBL_MAX;
+    double mMin = std::numeric_limits<double>::lowest() + 1;
+    double mMax = std::numeric_limits<double>::max();
     Type mDataType = Integer;
 };
 
+/**
+ * \class QgsProcessingParameterDistance
+ * \ingroup core
+ * A double numeric parameter for distance values. Linked to a source layer or CRS parameter
+ * to determine what units the distance values are in.
+ *
+ * The number of decimals places shown in a distance parameter's widget can be specified by
+ * setting the parameter's metadata. For example:
+ *
+ * * \code{.py}
+ *   param = QgsProcessingParameterDistance( 'VAL', 'Threshold')
+ *   # only show two decimal places in parameter's widgets, not 6:
+ *   param.setMetadata( {'widget_wrapper':
+ *     { 'decimals': 2 }
+ *   })
+ * \endcode
+ *
+ * \since QGIS 3.2
+ */
+class CORE_EXPORT QgsProcessingParameterDistance : public QgsProcessingParameterNumber
+{
+  public:
+
+    /**
+     * Constructor for QgsProcessingParameterDistance.
+     */
+    explicit QgsProcessingParameterDistance( const QString &name, const QString &description = QString(),
+        const QVariant &defaultValue = QVariant(),
+        const QString &parentParameterName = QString(),
+        bool optional = false,
+        double minValue = std::numeric_limits<double>::lowest() + 1,
+        double maxValue = std::numeric_limits<double>::max() );
+
+    /**
+     * Returns the type name for the parameter class.
+     */
+    static QString typeName() { return QStringLiteral( "distance" ); }
+
+    QgsProcessingParameterDistance *clone() const override SIP_FACTORY;
+
+    QString type() const override;
+    QStringList dependsOnOtherParameters() const override;
+
+    /**
+     * Returns the name of the parent parameter, or an empty string if this is not set.
+     * \see setParentParameterName()
+     */
+    QString parentParameterName() const;
+
+    /**
+     * Sets the name of the parent layer parameter. Use an empty string if this is not required.
+     * \see parentParameterName()
+     */
+    void setParentParameterName( const QString &parentParameterName );
+
+    /**
+     * Returns the default distance unit for the parameter.
+     *
+     * \see setDefaultUnit()
+     * \since QGIS 3.4.3
+     */
+    QgsUnitTypes::DistanceUnit defaultUnit() const { return mDefaultUnit; }
+
+    /**
+     * Sets the default distance \a unit for the parameter.
+     *
+     * \see defaultUnit()
+     * \since QGIS 3.4.3
+     */
+    void setDefaultUnit( QgsUnitTypes::DistanceUnit unit ) { mDefaultUnit = unit; }
+
+    QVariantMap toVariantMap() const override;
+    bool fromVariantMap( const QVariantMap &map ) override;
+
+  private:
+
+    QString mParentParameterName;
+    QgsUnitTypes::DistanceUnit mDefaultUnit = QgsUnitTypes::DistanceUnknownUnit;
+
+};
+
 /**
  * \class QgsProcessingParameterRange
  * \ingroup core
@@ -1703,9 +2007,12 @@ class CORE_EXPORT QgsProcessingDestinationParameter : public QgsProcessingParame
 
     /**
      * Constructor for QgsProcessingDestinationParameter.
+     *
+     * If \a createByDefault is false and the parameter is \a optional, then the destination
+     * output will not be created by default.
      */
     QgsProcessingDestinationParameter( const QString &name, const QString &description = QString(), const QVariant &defaultValue = QVariant(),
-                                       bool optional = false );
+                                       bool optional = false, bool createByDefault = true );
 
     bool isDestination() const override { return true; }
     QVariantMap toVariantMap() const override;
@@ -1758,11 +2065,34 @@ class CORE_EXPORT QgsProcessingDestinationParameter : public QgsProcessingParame
      */
     void setCreateByDefault( bool createByDefault );
 
+  protected:
+
+    /**
+     * Original (source) provider which this parameter has been derived from.
+     * In the case of destination parameters which are part of model algorithms, this
+     * will reflect the child algorithm's provider which actually generates the
+     * parameter, as opposed to the provider which this parameter belongs to (i.e.
+     * the model provider)
+     * \since QGIS 3.2
+     */
+    QgsProcessingProvider *originalProvider() const { return mOriginalProvider; }
+
   private:
 
+    /**
+     * Original (source) provider which this parameter has been derived from.
+     * In the case of destination parameters which are part of model algorithms, this
+     * will reflect the child algorithm's provider which actually generates the
+     * parameter, as opposed to the provider which this parameter belongs to (i.e.
+     * the model provider)
+     */
+    QgsProcessingProvider *mOriginalProvider = nullptr;
+
     bool mSupportsNonFileBasedOutputs = true;
     bool mCreateByDefault = true;
 
+    friend class QgsProcessingModelAlgorithm;
+    friend class TestQgsProcessing;
 };
 
 
@@ -1780,9 +2110,12 @@ class CORE_EXPORT QgsProcessingParameterFeatureSink : public QgsProcessingDestin
 
     /**
      * Constructor for QgsProcessingParameterFeatureSink.
+     *
+     * If \a createByDefault is false and the parameter is \a optional, then this destination
+     * output will not be created by default.
      */
     QgsProcessingParameterFeatureSink( const QString &name, const QString &description = QString(), QgsProcessing::SourceType type = QgsProcessing::TypeVectorAnyGeometry, const QVariant &defaultValue = QVariant(),
-                                       bool optional = false );
+                                       bool optional = false, bool createByDefault = true );
 
     /**
      * Returns the type name for the parameter class.
@@ -1796,6 +2129,13 @@ class CORE_EXPORT QgsProcessingParameterFeatureSink : public QgsProcessingDestin
     QgsProcessingOutputDefinition *toOutputDefinition() const override SIP_FACTORY;
     QString defaultFileExtension() const override;
 
+    /**
+     * Returns a list of the vector format file extensions supported by this parameter.
+     * \see defaultFileExtension()
+     * \since QGIS 3.2
+     */
+    virtual QStringList supportedOutputVectorLayerExtensions() const;
+
     /**
      * Returns the layer type for sinks associated with the parameter.
      * \see setDataType()
@@ -1845,9 +2185,12 @@ class CORE_EXPORT QgsProcessingParameterVectorDestination : public QgsProcessing
 
     /**
      * Constructor for QgsProcessingParameterVectorDestination.
+     *
+     * If \a createByDefault is false and the parameter is \a optional, then this destination
+     * output will not be created by default.
      */
     QgsProcessingParameterVectorDestination( const QString &name, const QString &description = QString(), QgsProcessing::SourceType type = QgsProcessing::TypeVectorAnyGeometry, const QVariant &defaultValue = QVariant(),
-        bool optional = false );
+        bool optional = false, bool createByDefault = true );
 
     /**
      * Returns the type name for the parameter class.
@@ -1861,6 +2204,13 @@ class CORE_EXPORT QgsProcessingParameterVectorDestination : public QgsProcessing
     QgsProcessingOutputDefinition *toOutputDefinition() const override SIP_FACTORY;
     QString defaultFileExtension() const override;
 
+    /**
+     * Returns a list of the vector format file extensions supported by this parameter.
+     * \see defaultFileExtension()
+     * \since QGIS 3.2
+     */
+    virtual QStringList supportedOutputVectorLayerExtensions() const;
+
     /**
      * Returns the layer type for this created vector layer.
      * \see setDataType()
@@ -1906,10 +2256,14 @@ class CORE_EXPORT QgsProcessingParameterRasterDestination : public QgsProcessing
 
     /**
      * Constructor for QgsProcessingParameterRasterDestination.
+     *
+     * If \a createByDefault is false and the parameter is \a optional, then this destination
+     * output will not be created by default.
      */
     QgsProcessingParameterRasterDestination( const QString &name, const QString &description = QString(),
         const QVariant &defaultValue = QVariant(),
-        bool optional = false );
+        bool optional = false,
+        bool createByDefault = true );
 
     /**
      * Returns the type name for the parameter class.
@@ -1922,6 +2276,13 @@ class CORE_EXPORT QgsProcessingParameterRasterDestination : public QgsProcessing
     QgsProcessingOutputDefinition *toOutputDefinition() const override SIP_FACTORY;
     QString defaultFileExtension() const override;
 
+    /**
+     * Returns a list of the raster format file extensions supported for this parameter.
+     * \see defaultFileExtension()
+     * \since QGIS 3.2
+     */
+    virtual QStringList supportedOutputRasterLayerExtensions() const;
+
     /**
      * Creates a new parameter using the definition from a script code.
      */
@@ -1941,11 +2302,15 @@ class CORE_EXPORT QgsProcessingParameterFileDestination : public QgsProcessingDe
 
     /**
      * Constructor for QgsProcessingParameterFileDestination.
+     *
+     * If \a createByDefault is false and the parameter is \a optional, then this destination
+     * output will not be created by default.
      */
     QgsProcessingParameterFileDestination( const QString &name, const QString &description = QString(),
                                            const QString &fileFilter = QString(),
                                            const QVariant &defaultValue = QVariant(),
-                                           bool optional = false );
+                                           bool optional = false,
+                                           bool createByDefault = true );
 
     /**
      * Returns the type name for the parameter class.
@@ -2035,7 +2400,8 @@ class CORE_EXPORT QgsProcessingParameterBand : public QgsProcessingParameterDefi
      */
     QgsProcessingParameterBand( const QString &name, const QString &description = QString(), const QVariant &defaultValue = QVariant(),
                                 const QString &parentLayerParameterName = QString(),
-                                bool optional = false );
+                                bool optional = false,
+                                bool allowMultiple = false );
 
     /**
      * Returns the type name for the parameter class.
@@ -2068,9 +2434,24 @@ class CORE_EXPORT QgsProcessingParameterBand : public QgsProcessingParameterDefi
      */
     static QgsProcessingParameterBand *fromScriptCode( const QString &name, const QString &description, bool isOptional, const QString &definition ) SIP_FACTORY;
 
+    /**
+     * Returns whether multiple band selections are permitted.
+     * \see setAllowMultiple()
+     * \since QGIS 3.4
+     */
+    bool allowMultiple() const;
+
+    /**
+     * Sets whether multiple band selections are permitted.
+     * \see allowMultiple()
+     * \since QGIS 3.4
+     */
+    void setAllowMultiple( bool allowMultiple );
+
   private:
 
     QString mParentLayerParameterName;
+    bool mAllowMultiple = false;
 };
 
 // clazy:excludeall=qstring-allocations
