@@ -23,7 +23,7 @@ __copyright__ = '(C) 2013, Alexander Bruy'
 
 # This will get replaced with a git SHA1 when you do a git archive
 
-__revision__ = '8a899c8758b30adc3f8d6df682a5702dd3bfe6f0'
+__revision__ = 'f6ddc62fdb8ae589d3827e84d0c25bf669931040'
 
 import os
 
@@ -32,8 +32,10 @@ from qgis.PyQt.QtGui import QIcon
 from osgeo import gdal, osr
 
 from processing.algs.gdal.GdalAlgorithm import GdalAlgorithm
-from processing.core.parameters import ParameterRaster
-from processing.core.parameters import ParameterBoolean
+from qgis.core import QgsProcessingException
+from qgis.core import (QgsProcessingParameterRasterLayer,
+                       QgsProcessingParameterBoolean,
+                       QgsProcessingOutputFile)
 
 pluginPath = os.path.split(os.path.split(os.path.dirname(__file__))[0])[0]
 
@@ -41,15 +43,26 @@ pluginPath = os.path.split(os.path.split(os.path.dirname(__file__))[0])[0]
 class ExtractProjection(GdalAlgorithm):
 
     INPUT = 'INPUT'
+    PRJ_FILE_CREATE = 'PRJ_FILE_CREATE'
+    WORLD_FILE = 'WORLD_FILE'
     PRJ_FILE = 'PRJ_FILE'
 
     def __init__(self):
         super().__init__()
 
     def initAlgorithm(self, config=None):
-        self.addParameter(ParameterRaster(self.INPUT, self.tr('Input file')))
-        self.addParameter(ParameterBoolean(self.PRJ_FILE,
-                                           self.tr('Create also .prj file'), False))
+        self.addParameter(QgsProcessingParameterRasterLayer(
+            self.INPUT,
+            self.tr('Input file')))
+        self.addParameter(QgsProcessingParameterBoolean(
+            self.PRJ_FILE_CREATE,
+            self.tr('Create also .prj file'), False))
+        self.addOutput(QgsProcessingOutputFile(
+            self.WORLD_FILE,
+            self.tr('World file')))
+        self.addOutput(QgsProcessingOutputFile(
+            self.PRJ_FILE,
+            self.tr('ESRI Shapefile prj file')))
 
     def name(self):
         return 'extractprojection'
@@ -58,7 +71,8 @@ class ExtractProjection(GdalAlgorithm):
         return self.tr('Extract projection')
 
     def icon(self):
-        return QIcon(os.path.join(pluginPath, 'images', 'gdaltools', 'projection-export.png'))
+        return QIcon(os.path.join(pluginPath, 'images', 'gdaltools',
+                                  'projection-export.png'))
 
     def group(self):
         return self.tr('Raster projections')
@@ -66,20 +80,34 @@ class ExtractProjection(GdalAlgorithm):
     def groupId(self):
         return 'rasterprojections'
 
-    def getConsoleCommands(self, parameters, context, feedback, executing=True):
-        return ["extractprojection"]
+    def commandName(self):
+        return 'extractprojection'
+
+    def getConsoleCommands(self, parameters, context, feedback,
+                           executing=True):
+        return [self.commandName()]
 
     def processAlgorithm(self, parameters, context, feedback):
-        rasterPath = self.getParameterValue(self.INPUT)
-        createPrj = self.getParameterValue(self.PRJ_FILE)
-
-        raster = gdal.Open(str(rasterPath))
-        crs = raster.GetProjection()
-        geotransform = raster.GetGeoTransform()
+        createPrj = self.parameterAsBool(parameters,
+                                         self.PRJ_FILE_CREATE,
+                                         context)
+        raster = self.parameterAsRasterLayer(parameters, self.INPUT,
+                                             context)
+        if not raster.dataProvider().name() == 'gdal':
+            raise QgsProcessingException(self.tr('This algorithm can '
+                                                 'only be used with '
+                                                 'GDAL raster layers'))
+        rasterPath = raster.source()
+        rasterDS = gdal.Open(rasterPath, gdal.GA_ReadOnly)
+        geotransform = rasterDS.GetGeoTransform()
+        inputcrs = raster.crs()
+        crs = inputcrs.toWkt()
         raster = None
+        rasterDS = None
 
         outFileName = os.path.splitext(str(rasterPath))[0]
 
+        results = {}
         if crs != '' and createPrj:
             tmp = osr.SpatialReference()
             tmp.ImportFromWkt(crs)
@@ -89,15 +117,21 @@ class ExtractProjection(GdalAlgorithm):
 
             with open(outFileName + '.prj', 'wt') as prj:
                 prj.write(crs)
+            results[self.PRJ_FILE] = outFileName + '.prj'
+        else:
+            results[self.PRJ_FILE] = None
 
         with open(outFileName + '.wld', 'wt') as wld:
             wld.write('%0.8f\n' % geotransform[1])
             wld.write('%0.8f\n' % geotransform[4])
             wld.write('%0.8f\n' % geotransform[2])
             wld.write('%0.8f\n' % geotransform[5])
-            wld.write('%0.8f\n' % (geotransform[0] +
-                                   0.5 * geotransform[1] +
-                                   0.5 * geotransform[2]))
-            wld.write('%0.8f\n' % (geotransform[3] +
-                                   0.5 * geotransform[4] +
-                                   0.5 * geotransform[5]))
+            wld.write('%0.8f\n' % (geotransform[0]
+                                   + 0.5 * geotransform[1]
+                                   + 0.5 * geotransform[2]))
+            wld.write('%0.8f\n' % (geotransform[3]
+                                   + 0.5 * geotransform[4]
+                                   + 0.5 * geotransform[5]))
+        results[self.WORLD_FILE] = outFileName + '.wld'
+
+        return results
