@@ -23,7 +23,7 @@ __copyright__ = '(C) 2016, Alexander Bruy'
 
 # This will get replaced with a git SHA1 when you do a git archive
 
-__revision__ = '26842169e9367434a27a70ab0df0bd4ba6ad1d7a'
+__revision__ = 'f6ddc62fdb8ae589d3827e84d0c25bf669931040'
 
 from qgis.PyQt.QtWidgets import QLineEdit, QComboBox
 from qgis.gui import QgsRasterFormatSaveOptionsWidget
@@ -43,12 +43,12 @@ class RasterOptionsWidgetWrapper(WidgetWrapper):
             options = [(self.dialog.resolveValueDescription(s), s) for s in strings]
             for desc, val in options:
                 widget.addItem(desc, val)
-            widget.setEditText(self.param.defaultValue() or '')
+            widget.setEditText(self.parameterDefinition().defaultValue() or '')
             return widget
         elif self.dialogType == DIALOG_BATCH:
             widget = QLineEdit()
-            if self.param.defaultValue():
-                widget.setText(self.param.defaultValue())
+            if self.parameterDefinition().defaultValue():
+                widget.setText(self.parameterDefinition().defaultValue())
             return widget
         else:
             return QgsRasterFormatSaveOptionsWidget()
