@@ -23,7 +23,7 @@ __copyright__ = '(C) 2012-2015, Victor Olaya'
 
 # This will get replaced with a git SHA1 when you do a git archive
 
-__revision__ = 'a86bec25ebddd442bde3c6fe8ee761b22b36c26c'
+__revision__ = 'f6ddc62fdb8ae589d3827e84d0c25bf669931040'
 
 import sys
 import os
@@ -51,6 +51,7 @@ from qgis.core import (Qgis,
                        QgsProcessingParameterField,
                        QgsProcessingParameterPoint,
                        QgsProcessingParameterBoolean,
+                       QgsProcessingParameterFeatureSource,
                        QgsProcessingParameterVectorLayer,
                        QgsProcessingParameterRasterLayer,
                        QgsProcessingParameterMultipleLayers,
@@ -59,8 +60,11 @@ from qgis.core import (Qgis,
                        QgsProcessingParameterFileDestination,
                        QgsProcessingParameterFile,
                        QgsProcessingParameterFolderDestination,
-                       QgsProcessingUtils)
+                       QgsProcessingOutputHtml,
+                       QgsVectorLayer,
+                       QgsProviderRegistry)
 from qgis.utils import iface
+from osgeo import ogr
 
 from processing.core.ProcessingConfig import ProcessingConfig
 
@@ -68,7 +72,6 @@ from processing.core.parameters import (getParameterFromString)
 
 from .Grass7Utils import Grass7Utils
 
-#from processing.tools import dataobjects, system
 from processing.tools.system import isWindows, getTempFilename
 
 pluginPath = os.path.normpath(os.path.join(
@@ -85,6 +88,8 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
     GRASS_REGION_ALIGN_TO_RESOLUTION = 'GRASS_REGION_ALIGN_TO_RESOLUTION'
     GRASS_RASTER_FORMAT_OPT = 'GRASS_RASTER_FORMAT_OPT'
     GRASS_RASTER_FORMAT_META = 'GRASS_RASTER_FORMAT_META'
+    GRASS_VECTOR_DSCO = 'GRASS_VECTOR_DSCO'
+    GRASS_VECTOR_LCO = 'GRASS_VECTOR_LCO'
 
     OUTPUT_TYPES = ['auto', 'point', 'line', 'area']
     QGIS_OUTPUT_TYPES = {QgsProcessing.TypeVectorAnyGeometry: 'auto',
@@ -96,19 +101,23 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
         super().__init__()
         self._name = ''
         self._display_name = ''
+        self._short_description = ''
         self._group = ''
         self._groupId = ''
-        self.groupIdRegex = re.compile('^[^\s\(]+')
+        self.groupIdRegex = re.compile(r'^[^\s\(]+')
         self.grass7Name = ''
         self.params = []
         self.hardcodedStrings = []
         self.inputLayers = []
+        self.commands = []
+        self.outputCommands = []
+        self.exportedLayers = {}
         self.descriptionFile = descriptionfile
 
         # Default GRASS parameters
         self.region = None
         self.cellSize = None
-        self.snaptTolerance = None
+        self.snapTolerance = None
         self.outputType = None
         self.minArea = None
         self.alignToResolution = None
@@ -136,6 +145,9 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
     def displayName(self):
         return self._display_name
 
+    def shortDescription(self):
+        return self._short_description
+
     def group(self):
         return self._group
 
@@ -150,7 +162,7 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
 
     def flags(self):
         # TODO - maybe it's safe to background thread this?
-        return super().flags() | QgsProcessingAlgorithm.FlagNoThreading
+        return super().flags() | QgsProcessingAlgorithm.FlagNoThreading | QgsProcessingAlgorithm.FlagDisplayNameIsLiteral
 
     def tr(self, string, context=''):
         if context == '':
@@ -173,7 +185,7 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
         """
         for p in self.params:
             # We use createOutput argument for automatic output creation
-            res = self.addParameter(p, True)
+            self.addParameter(p, True)
 
     def defineCharacteristicsFromFile(self):
         """
@@ -185,13 +197,12 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
             self.grass7Name = line
             # Second line if the algorithm name in Processing
             line = lines.readline().strip('\n').strip()
-            self._name = line
-            self._display_name = QCoreApplication.translate("GrassAlgorithm", line)
-            if " - " not in self._name:
-                self._name = self.grass7Name + " - " + self._name
-                self._display_name = self.grass7Name + " - " + self._display_name
-
-            self._name = self._name[:self._name.find(' ')].lower()
+            self._short_description = line
+            if " - " not in line:
+                self._name = self.grass7Name
+            else:
+                self._name = line[:line.find(' ')].lower()
+            self._display_name = self._name
             # Read the grass group
             line = lines.readline().strip('\n').strip()
             self._group = QCoreApplication.translate("GrassAlgorithm", line)
@@ -210,7 +221,7 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
                     parameter = getParameterFromString(line)
                     if parameter is not None:
                         self.params.append(parameter)
-                        if isinstance(parameter, QgsProcessingParameterVectorLayer):
+                        if isinstance(parameter, (QgsProcessingParameterVectorLayer, QgsProcessingParameterFeatureSource)):
                             hasVectorInput = True
                         elif isinstance(parameter, QgsProcessingParameterRasterLayer):
                             hasRasterInput = True
@@ -283,12 +294,31 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
             self.params.append(param)
 
         if vectorOutputs:
+            # Add an optional output type
             param = QgsProcessingParameterEnum(self.GRASS_OUTPUT_TYPE_PARAMETER,
                                                self.tr('v.out.ogr output type'),
                                                self.OUTPUT_TYPES)
             param.setFlags(param.flags() | QgsProcessingParameterDefinition.FlagAdvanced)
             self.params.append(param)
 
+            # Add a DSCO parameter for format export
+            param = QgsProcessingParameterString(
+                self.GRASS_VECTOR_DSCO,
+                self.tr('v.out.ogr output data source options (dsco)'),
+                multiLine=True, optional=True
+            )
+            param.setFlags(param.flags() | QgsProcessingParameterDefinition.FlagAdvanced)
+            self.params.append(param)
+
+            # Add a LCO parameter for format export
+            param = QgsProcessingParameterString(
+                self.GRASS_VECTOR_LCO,
+                self.tr('v.out.ogr output layer options (lco)'),
+                multiLine=True, optional=True
+            )
+            param.setFlags(param.flags() | QgsProcessingParameterDefinition.FlagAdvanced)
+            self.params.append(param)
+
     def getDefaultCellSize(self):
         """
         Determine a default cell size from all the raster layers.
@@ -335,7 +365,7 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
                                                       self.GRASS_REGION_ALIGN_TO_RESOLUTION,
                                                       context)
 
-    def processAlgorithm(self, parameters, context, feedback):
+    def processAlgorithm(self, original_parameters, context, feedback):
         if isWindows():
             path = Grass7Utils.grassPath()
             if path == '':
@@ -343,6 +373,9 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
                     self.tr('GRASS GIS 7 folder is not configured. Please '
                             'configure it before running GRASS GIS 7 algorithms.'))
 
+        # make a copy of the original parameters dictionary - it gets modified by grass algorithms
+        parameters = {k: v for k, v in original_parameters.items()}
+
         # Create brand new commands lists
         self.commands = []
         self.outputCommands = []
@@ -364,9 +397,9 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
         for fName in ['Inputs', 'Command', 'Outputs']:
             fullName = 'process{}'.format(fName)
             if self.module and hasattr(self.module, fullName):
-                getattr(self.module, fullName)(self, parameters, context)
+                getattr(self.module, fullName)(self, parameters, context, feedback)
             else:
-                getattr(self, fullName)(parameters, context)
+                getattr(self, fullName)(parameters, context, feedback)
 
         # Run GRASS
         loglines = []
@@ -397,15 +430,16 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
 
         return outputs
 
-    def processInputs(self, parameters, context):
+    def processInputs(self, parameters, context, feedback):
         """Prepare the GRASS import commands"""
         inputs = [p for p in self.parameterDefinitions()
                   if isinstance(p, (QgsProcessingParameterVectorLayer,
+                                    QgsProcessingParameterFeatureSource,
                                     QgsProcessingParameterRasterLayer,
                                     QgsProcessingParameterMultipleLayers))]
         for param in inputs:
             paramName = param.name()
-            if not paramName in parameters:
+            if paramName not in parameters:
                 continue
             # Handle Null parameter
             if parameters[paramName] is None:
@@ -419,7 +453,7 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
                     self.loadRasterLayerFromParameter(
                         paramName, parameters, context)
             # Vector inputs needs to be imported into temp GRASS DB
-            elif isinstance(param, QgsProcessingParameterVectorLayer):
+            elif isinstance(param, (QgsProcessingParameterFeatureSource, QgsProcessingParameterVectorLayer)):
                 if paramName not in self.exportedLayers:
                     # Attribute tables are also vector inputs
                     if QgsProcessing.TypeFile in param.dataTypes():
@@ -427,7 +461,7 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
                             paramName, parameters, context)
                     else:
                         self.loadVectorLayerFromParameter(
-                            paramName, parameters, context, None)
+                            paramName, parameters, context, external=None, feedback=feedback)
             # For multiple inputs, process each layer
             elif isinstance(param, QgsProcessingParameterMultipleLayers):
                 layers = self.parameterAsLayerList(parameters, paramName, context)
@@ -438,7 +472,7 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
                         self.loadRasterLayer(layerName, layer)
                     # Add a vector layer
                     elif layer.type() == QgsMapLayer.VectorLayer:
-                        self.loadVectorLayer(layerName, layer, None)
+                        self.loadVectorLayer(layerName, layer, external=None, feedback=feedback)
 
         self.postInputs()
 
@@ -473,7 +507,7 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
 
         QgsMessageLog.logMessage(self.tr('processInputs end. Commands: {}').format(self.commands), 'Grass7', Qgis.Info)
 
-    def processCommand(self, parameters, context, delOutputs=False):
+    def processCommand(self, parameters, context, feedback, delOutputs=False):
         """
         Prepare the GRASS algorithm command
         :param parameters:
@@ -497,12 +531,15 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
                              self.GRASS_OUTPUT_TYPE_PARAMETER,
                              self.GRASS_REGION_ALIGN_TO_RESOLUTION,
                              self.GRASS_RASTER_FORMAT_OPT,
-                             self.GRASS_RASTER_FORMAT_META]:
+                             self.GRASS_RASTER_FORMAT_META,
+                             self.GRASS_VECTOR_DSCO,
+                             self.GRASS_VECTOR_LCO]:
                 continue
 
             # Raster and vector layers
             if isinstance(param, (QgsProcessingParameterRasterLayer,
-                                  QgsProcessingParameterVectorLayer)):
+                                  QgsProcessingParameterVectorLayer,
+                                  QgsProcessingParameterFeatureSource)):
                 if paramName in self.exportedLayers:
                     value = self.exportedLayers[paramName]
                 else:
@@ -569,7 +606,7 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
                     self.parameterAsString(parameters, paramName, context)
                 )
             if value:
-                command += ' {}={}'.format(paramName, value)
+                command += ' {}={}'.format(paramName.replace('~', ''), value)
 
         # Handle outputs
         if not delOutputs:
@@ -580,32 +617,33 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
                 outName = out.name()
                 # For File destination
                 if isinstance(out, QgsProcessingParameterFileDestination):
-                    # for HTML reports, we need to redirect stdout
-                    if out.defaultFileExtension().lower() == 'html':
-                        command += ' > "{}"'.format(
-                            self.parameterAsFileOutput(
-                                parameters, outName, context)
-                        )
-                    else:
-                        command += ' {}="{}"'.format(
-                            outName,
-                            self.parameterAsFileOutput(
-                                parameters, outName, context))
+                    if outName in parameters and parameters[outName] is not None:
+                        # for HTML reports, we need to redirect stdout
+                        if out.defaultFileExtension().lower() == 'html':
+                            command += ' > "{}"'.format(
+                                self.parameterAsFileOutput(
+                                    parameters, outName, context)
+                            )
+                        else:
+                            command += ' {}="{}"'.format(
+                                outName,
+                                self.parameterAsFileOutput(
+                                    parameters, outName, context))
                 # For folders destination
                 elif isinstance(out, QgsProcessingParameterFolderDestination):
                     # We need to add a unique temporary basename
                     uniqueBasename = outName + self.uniqueSuffix
                     command += ' {}={}'.format(outName, uniqueBasename)
                 else:
-                    # We add an output name to make sure it is unique if the session
-                    # uses this algorithm several times.
-                    #value = self.parameterAsOutputLayer(parameters, outName, context)
-                    uniqueOutputName = outName + self.uniqueSuffix
-                    command += ' {}={}'.format(outName, uniqueOutputName)
+                    if outName in parameters and parameters[outName] is not None:
+                        # We add an output name to make sure it is unique if the session
+                        # uses this algorithm several times.
+                        uniqueOutputName = outName + self.uniqueSuffix
+                        command += ' {}={}'.format(outName, uniqueOutputName)
 
-                    # Add output file to exported layers, to indicate that
-                    # they are present in GRASS
-                    self.exportedLayers[outName] = uniqueOutputName
+                        # Add output file to exported layers, to indicate that
+                        # they are present in GRASS
+                        self.exportedLayers[outName] = uniqueOutputName
 
         command += ' --overwrite'
         self.commands.append(command)
@@ -621,16 +659,17 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
             self.outType = ('auto' if typeidx
                             is None else self.OUTPUT_TYPES[typeidx])
 
-    def processOutputs(self, parameters, context):
+    def processOutputs(self, parameters, context, feedback):
         """Prepare the GRASS v.out.ogr commands"""
-        # TODO: support multiple raster formats.
-        # TODO: support multiple vector formats.
-
         # Determine general vector output type
         self.vectorOutputType(parameters, context)
 
         for out in self.destinationParameterDefinitions():
             outName = out.name()
+            if outName not in parameters:
+                # skipped output
+                continue
+
             if isinstance(out, QgsProcessingParameterRasterDestination):
                 self.exportRasterLayerFromParameter(outName, parameters, context)
             elif isinstance(out, QgsProcessingParameterVectorDestination):
@@ -682,8 +721,11 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
         :param context: Algorithm context.
         :param colorTable: preserve color Table.
         """
-        fileName = os.path.normpath(
-            self.parameterAsOutputLayer(parameters, name, context))
+        fileName = self.parameterAsOutputLayer(parameters, name, context)
+        if not fileName:
+            return
+
+        fileName = os.path.normpath(fileName)
         grassName = '{}{}'.format(name, self.uniqueSuffix)
         outFormat = Grass7Utils.getRasterFormatFromFilename(fileName)
         createOpt = self.parameterAsString(parameters, self.GRASS_RASTER_FORMAT_OPT, context)
@@ -702,7 +744,7 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
         :param colorTable: preserve color Table.
         :param outFormat: file format for export.
         :param createOpt: creation options for format.
-        :param metatOpt: metadata options for export.
+        :param metaOpt: metadata options for export.
         """
         if not createOpt:
             if outFormat in Grass7Utils.GRASS_RASTER_FORMATS_CREATEOPTS:
@@ -740,17 +782,24 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
 
         # Add a loop export from the basename
         for cmd in [self.commands, self.outputCommands]:
-            # TODO Windows support
             # TODO Format/options support
-            cmd.append("for r in $(g.list type=rast pattern='{}*'); do".format(basename))
-            cmd.append("  r.out.gdal -m{0} input=${{r}} output={1}/${{r}}.tif {2}".format(
-                ' -t' if colorTable else '', outDir,
-                '--overwrite -c createopt="TFW=YES,COMPRESS=LZW"'
-            )
-            )
-            cmd.append("done")
+            if isWindows():
+                cmd.append("if not exist {0} mkdir {0}".format(outDir))
+                cmd.append("for /F %%r IN ('g.list type^=rast pattern^=\"{0}*\"') do r.out.gdal -m{1} input=%%r output={2}/%%r.tif {3}".format(
+                    basename,
+                    ' -t' if colorTable else '',
+                    outDir,
+                    '--overwrite -c createopt="TFW=YES,COMPRESS=LZW"'
+                ))
+            else:
+                cmd.append("for r in $(g.list type=rast pattern='{}*'); do".format(basename))
+                cmd.append("  r.out.gdal -m{0} input=${{r}} output={1}/${{r}}.tif {2}".format(
+                    ' -t' if colorTable else '', outDir,
+                    '--overwrite -c createopt="TFW=YES,COMPRESS=LZW"'
+                ))
+                cmd.append("done")
 
-    def loadVectorLayerFromParameter(self, name, parameters, context, external=False):
+    def loadVectorLayerFromParameter(self, name, parameters, context, feedback, external=False):
         """
         Creates a dedicated command to load a vector into
         the temporary GRASS DB.
@@ -760,43 +809,89 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
         :param external: use v.external (v.in.ogr if False).
         """
         layer = self.parameterAsVectorLayer(parameters, name, context)
-        self.loadVectorLayer(name, layer, external)
 
-    def loadVectorLayer(self, name, layer, external=False):
+        is_ogr_disk_based_layer = layer is not None and layer.dataProvider().name() == 'ogr'
+        if is_ogr_disk_based_layer:
+            # we only support direct reading of disk based ogr layers -- not ogr postgres layers, etc
+            source_parts = QgsProviderRegistry.instance().decodeUri('ogr', layer.source())
+            if not source_parts.get('path'):
+                is_ogr_disk_based_layer = False
+            elif source_parts.get('layerId'):
+                # no support for directly reading layers by id in grass
+                is_ogr_disk_based_layer = False
+
+        if not is_ogr_disk_based_layer:
+            # parameter is not a vector layer or not an OGR layer - try to convert to a source compatible with
+            # grass OGR inputs and extract selection if required
+            path = self.parameterAsCompatibleSourceLayerPath(parameters, name, context,
+                                                             QgsVectorFileWriter.supportedFormatExtensions(),
+                                                             feedback=feedback)
+            ogr_layer = QgsVectorLayer(path, '', 'ogr')
+            self.loadVectorLayer(name, ogr_layer, external=external, feedback=feedback)
+        else:
+            # already an ogr disk based layer source
+            self.loadVectorLayer(name, layer, external=external, feedback=feedback)
+
+    def loadVectorLayer(self, name, layer, external=False, feedback=None):
         """
         Creates a dedicated command to load a vector into
         temporary GRASS DB.
         :param name: name of the parameter
         :param layer: QgsMapLayer for the vector layer.
         :param external: use v.external (v.in.ogr if False).
+        :param feedback: feedback object
         """
-        # TODO: support selections
         # TODO: support multiple input formats
         if external is None:
             external = ProcessingConfig.getSetting(
                 Grass7Utils.GRASS_USE_VEXTERNAL)
+
+        source_parts = QgsProviderRegistry.instance().decodeUri('ogr', layer.source())
+        file_path = source_parts.get('path')
+        layer_name = source_parts.get('layerName')
+
+        # safety check: we can only use external for ogr layers which support random read
+        if external:
+            if feedback is not None:
+                feedback.pushInfo('Attempting to use v.external for direct layer read')
+            ds = ogr.Open(file_path)
+            if ds is not None:
+                ogr_layer = ds.GetLayer()
+                if ogr_layer is None or not ogr_layer.TestCapability(ogr.OLCRandomRead):
+                    if feedback is not None:
+                        feedback.reportError('Cannot use v.external: layer does not support random read')
+                    external = False
+            else:
+                if feedback is not None:
+                    feedback.reportError('Cannot use v.external: error reading layer')
+                external = False
+
         self.inputLayers.append(layer)
         self.setSessionProjectionFromLayer(layer)
         destFilename = 'vector_{}'.format(os.path.basename(getTempFilename()))
         self.exportedLayers[name] = destFilename
-        command = '{0}{1}{2} input="{3}" output="{4}" --overwrite -o'.format(
+        command = '{0}{1}{2} input="{3}"{4} output="{5}" --overwrite -o'.format(
             'v.external' if external else 'v.in.ogr',
             ' min_area={}'.format(self.minArea) if not external else '',
             ' snap={}'.format(self.snapTolerance) if not external else '',
-            os.path.normpath(layer.source()),
+            os.path.normpath(file_path),
+            ' layer="{}"'.format(layer_name) if layer_name else '',
             destFilename)
         self.commands.append(command)
 
-    def exportVectorLayerFromParameter(self, name, parameters, context):
+    def exportVectorLayerFromParameter(self, name, parameters, context, layer=None, nocats=False):
         """
-        Creates a dedicated command to export a raster from
-        temporary GRASS DB into a file via gdal.
-        :param grassName: name of the parameter
-        :param fileName: file path of raster layer
-        :param colorTable: preserve color Table.
+        Creates a dedicated command to export a vector from
+        a QgsProcessingParameter.
+        :param name: name of the parameter.
+        :param context: parameters context.
+        :param layer: for vector with multiples layers, exports only one layer.
+        :param nocats: do not export GRASS categories.
         """
         fileName = os.path.normpath(
             self.parameterAsOutputLayer(parameters, name, context))
+        grassName = '{}{}'.format(name, self.uniqueSuffix)
+
         # Find if there is a dataType
         dataType = self.outType
         if self.outType == 'auto':
@@ -806,28 +901,37 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
                 if layerType in self.QGIS_OUTPUT_TYPES:
                     dataType = self.QGIS_OUTPUT_TYPES[layerType]
 
-        grassName = '{}{}'.format(name, self.uniqueSuffix)
-        self.exportVectorLayer(grassName, fileName, dataType)
+        outFormat = QgsVectorFileWriter.driverForExtension(os.path.splitext(fileName)[1]).replace(' ', '_')
+        dsco = self.parameterAsString(parameters, self.GRASS_VECTOR_DSCO, context)
+        lco = self.parameterAsString(parameters, self.GRASS_VECTOR_LCO, context)
+        self.exportVectorLayer(grassName, fileName, layer, nocats, dataType, outFormat, dsco, lco)
 
-    def exportVectorLayer(self, grassName, fileName, dataType='auto', layer=None, nocats=False):
+    def exportVectorLayer(self, grassName, fileName, layer=None, nocats=False, dataType='auto',
+                          outFormat=None, dsco=None, lco=None):
         """
         Creates a dedicated command to export a vector from
-        temporary GRASS DB into a file via ogr.
-        :param grassName: name of the parameter.
-        :param fileName: file path of raster layer.
-        :param dataType: GRASS data type for exporting data.
-        :param layer: In GRASS a vector can have multiple layers.
-        :param nocats: Also export features without category if True.
+        temporary GRASS DB into a file via OGR.
+        :param grassName: name of the vector to export.
+        :param fileName: file path of vector layer.
+        :param dataType: export only this type of data.
+        :param layer: for vector with multiples layers, exports only one layer.
+        :param nocats: do not export GRASS categories.
+        :param outFormat: file format for export.
+        :param dsco: datasource creation options for format.
+        :param lco: layer creation options for format.
         """
+        if outFormat is None:
+            outFormat = QgsVectorFileWriter.driverForExtension(os.path.splitext(fileName)[1]).replace(' ', '_')
+
         for cmd in [self.commands, self.outputCommands]:
             cmd.append(
-                'v.out.ogr{0} type={1} {2} input="{3}" output="{4}" {5}'.format(
-                    ' -c' if nocats else '',
-                    dataType,
+                'v.out.ogr{0} type="{1}" input="{2}" output="{3}" format="{4}" {5}{6}{7} --overwrite'.format(
+                    '' if nocats else ' -c',
+                    dataType, grassName, fileName,
+                    outFormat,
                     'layer={}'.format(layer) if layer else '',
-                    grassName,
-                    fileName,
-                    'format=ESRI_Shapefile --overwrite'
+                    ' dsco="{}"'.format(dsco) if dsco else '',
+                    ' lco="{}"'.format(lco) if lco else ''
                 )
             )
 
@@ -899,7 +1003,6 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
     def convertToHtml(self, fileName):
         # Read HTML contents
         lines = []
-        html = False
         with open(fileName, 'r', encoding='utf-8') as f:
             lines = f.readlines()
 
@@ -918,9 +1021,9 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
         return not message, message
 
     def checkParameterValues(self, parameters, context):
+        grass_parameters = {k: v for k, v in parameters.items()}
         if self.module:
             if hasattr(self.module, 'checkParameterValuesBeforeExecuting'):
                 func = getattr(self.module, 'checkParameterValuesBeforeExecuting')
-                #return func(self, parameters, context), None
-                return None, func(self, parameters, context)
-        return super(Grass7Algorithm, self).checkParameterValues(parameters, context)
+                return func(self, grass_parameters, context)
+        return super().checkParameterValues(grass_parameters, context)
