@@ -23,7 +23,7 @@ __copyright__ = '(C) 2017, Nyall Dawson'
 
 # This will get replaced with a git SHA1 when you do a git archive323
 
-__revision__ = 'bc43194061452b37c8a22edcb7b380055a0c4236'
+__revision__ = 'f6ddc62fdb8ae589d3827e84d0c25bf669931040'
 
 import os
 
@@ -105,3 +105,6 @@ class SetZValue(QgisFeatureBasedAlgorithm):
             feature.setGeometry(QgsGeometry(new_geom))
 
         return [feature]
+
+    def supportInPlaceEdit(self, layer):
+        return super().supportInPlaceEdit(layer) and QgsWkbTypes.hasZ(layer.wkbType())
