@@ -23,9 +23,10 @@ __copyright__ = '(C) 2016, Nyall Dawson'
 
 # This will get replaced with a git SHA1 when you do a git archive
 
-__revision__ = '26842169e9367434a27a70ab0df0bd4ba6ad1d7a'
+__revision__ = 'f6ddc62fdb8ae589d3827e84d0c25bf669931040'
 
 from qgis.analysis import (QgsGeometrySnapper,
+                           QgsGeometrySnapperSingleSource,
                            QgsInternalGeometrySnapper)
 from qgis.core import (QgsFeatureSink,
                        QgsProcessing,
@@ -33,7 +34,6 @@ from qgis.core import (QgsFeatureSink,
                        QgsProcessingParameterDistance,
                        QgsProcessingParameterFeatureSource,
                        QgsProcessingParameterFeatureSink,
-                       QgsProcessingParameterNumber,
                        QgsProcessingParameterEnum)
 
 from processing.algs.qgis.QgisAlgorithm import QgisAlgorithm
@@ -63,8 +63,8 @@ class SnapGeometriesToLayer(QgisAlgorithm):
                                                                QgsProcessing.TypeVectorLine,
                                                                QgsProcessing.TypeVectorPolygon]))
 
-        self.addParameter(QgsProcessingParameterDistance(self.TOLERANCE, self.tr('Tolerance (layer units)'), parentParameterName=self.INPUT,
-                                                         minValue=0.00000001, maxValue=9999999999, defaultValue=10.0))
+        self.addParameter(QgsProcessingParameterDistance(self.TOLERANCE, self.tr('Tolerance'), parentParameterName=self.INPUT,
+                                                         minValue=0.00000001, defaultValue=10.0))
 
         self.modes = [self.tr('Prefer aligning nodes, insert extra vertices where required'),
                       self.tr('Prefer closest point, insert extra vertices where required'),
@@ -72,7 +72,8 @@ class SnapGeometriesToLayer(QgisAlgorithm):
                       self.tr('Prefer closest point, don\'t insert new vertices'),
                       self.tr('Move end points only, prefer aligning nodes'),
                       self.tr('Move end points only, prefer closest point'),
-                      self.tr('Snap end points to end points only')]
+                      self.tr('Snap end points to end points only'),
+                      self.tr('Snap to anchor nodes (single layer only)')]
         self.addParameter(QgsProcessingParameterEnum(
             self.BEHAVIOR,
             self.tr('Behavior'),
@@ -107,6 +108,9 @@ class SnapGeometriesToLayer(QgisAlgorithm):
         total = 100.0 / source.featureCount() if source.featureCount() else 0
 
         if parameters[self.INPUT] != parameters[self.REFERENCE_LAYER]:
+            if mode == 7:
+                raise QgsProcessingException(self.tr('This mode applies when the input and reference layer are the same.'))
+
             snapper = QgsGeometrySnapper(reference_source)
             processed = 0
             for f in features:
@@ -120,6 +124,10 @@ class SnapGeometriesToLayer(QgisAlgorithm):
                     sink.addFeature(f)
                 processed += 1
                 feedback.setProgress(processed * total)
+        elif mode == 7:
+            # input layer == ref layer
+            modified_count = QgsGeometrySnapperSingleSource.run(source, sink, tolerance, feedback)
+            feedback.pushInfo(self.tr('Snapped {} geometries.').format(modified_count))
         else:
             # snapping internally
             snapper = QgsInternalGeometrySnapper(tolerance, mode)
