@@ -23,7 +23,7 @@ __copyright__ = '(C) 2012, Victor Olaya'
 
 # This will get replaced with a git SHA1 when you do a git archive
 
-__revision__ = '26842169e9367434a27a70ab0df0bd4ba6ad1d7a'
+__revision__ = 'f6ddc62fdb8ae589d3827e84d0c25bf669931040'
 
 from pprint import pformat
 import time
@@ -43,8 +43,11 @@ from qgis.core import (Qgis,
                        QgsProcessingOutputLayerDefinition,
                        QgsProcessingParameterFeatureSink,
                        QgsProcessingParameterRasterDestination,
-                       QgsProcessingAlgorithm)
-from qgis.gui import (QgsMessageBar,
+                       QgsProcessingAlgorithm,
+                       QgsProxyProgressTask,
+                       QgsTaskManager)
+from qgis.gui import (QgsGui,
+                      QgsMessageBar,
                       QgsProcessingAlgorithmDialogBase)
 from qgis.utils import iface
 
@@ -54,31 +57,47 @@ from processing.core.ProcessingResults import resultsList
 from processing.gui.ParametersPanel import ParametersPanel
 from processing.gui.BatchAlgorithmDialog import BatchAlgorithmDialog
 from processing.gui.AlgorithmDialogBase import AlgorithmDialogBase
-from processing.gui.AlgorithmExecutor import executeIterating, execute
+from processing.gui.AlgorithmExecutor import executeIterating, execute, execute_in_place
 from processing.gui.Postprocessing import handleAlgorithmResults
+from processing.gui.wrappers import WidgetWrapper
 
 from processing.tools import dataobjects
 
 
 class AlgorithmDialog(QgsProcessingAlgorithmDialogBase):
 
-    def __init__(self, alg):
-        super().__init__()
+    def __init__(self, alg, in_place=False, parent=None):
+        super().__init__(parent)
+
         self.feedback_dialog = None
+        self.in_place = in_place
+        self.active_layer = None
+
+        self.context = None
+        self.feedback = None
 
         self.setAlgorithm(alg)
         self.setMainWidget(self.getParametersPanel(alg, self))
 
-        self.runAsBatchButton = QPushButton(QCoreApplication.translate("AlgorithmDialog", "Run as Batch Process…"))
-        self.runAsBatchButton.clicked.connect(self.runAsBatch)
-        self.buttonBox().addButton(self.runAsBatchButton, QDialogButtonBox.ResetRole) # reset role to ensure left alignment
+        if not self.in_place:
+            self.runAsBatchButton = QPushButton(QCoreApplication.translate("AlgorithmDialog", "Run as Batch Process…"))
+            self.runAsBatchButton.clicked.connect(self.runAsBatch)
+            self.buttonBox().addButton(self.runAsBatchButton, QDialogButtonBox.ResetRole) # reset role to ensure left alignment
+        else:
+            self.active_layer = iface.activeLayer()
+            self.runAsBatchButton = None
+            has_selection = self.active_layer and (self.active_layer.selectedFeatureCount() > 0)
+            self.buttonBox().button(QDialogButtonBox.Ok).setText(QCoreApplication.translate("AlgorithmDialog", "Modify Selected Features")
+                                                                 if has_selection else QCoreApplication.translate("AlgorithmDialog", "Modify All Features"))
+            self.buttonBox().button(QDialogButtonBox.Close).setText(QCoreApplication.translate("AlgorithmDialog", "Cancel"))
+            self.setWindowTitle(self.windowTitle() + ' | ' + self.active_layer.name())
 
     def getParametersPanel(self, alg, parent):
-        return ParametersPanel(parent, alg)
+        return ParametersPanel(parent, alg, self.in_place)
 
     def runAsBatch(self):
         self.close()
-        dlg = BatchAlgorithmDialog(self.algorithm())
+        dlg = BatchAlgorithmDialog(self.algorithm().create(), parent=iface.mainWindow())
         dlg.show()
         dlg.exec_()
 
@@ -95,15 +114,38 @@ class AlgorithmDialog(QgsProcessingAlgorithmDialogBase):
             if param.flags() & QgsProcessingParameterDefinition.FlagHidden:
                 continue
             if not param.isDestination():
-                wrapper = self.mainWidget().wrappers[param.name()]
-                value = None
-                if wrapper.widget is not None:
-                    value = wrapper.value()
-                    parameters[param.name()] = value
 
-                    if not param.checkValueIsAcceptable(value):
-                        raise AlgorithmDialogBase.InvalidParameterValue(param, wrapper.widget)
+                if self.in_place and param.name() == 'INPUT':
+                    parameters[param.name()] = self.active_layer
+                    continue
+
+                try:
+                    wrapper = self.mainWidget().wrappers[param.name()]
+                except KeyError:
+                    continue
+
+                # For compatibility with 3.x API, we need to check whether the wrapper is
+                # the deprecated WidgetWrapper class. If not, it's the newer
+                # QgsAbstractProcessingParameterWidgetWrapper class
+                # TODO QGIS 4.0 - remove
+                if issubclass(wrapper.__class__, WidgetWrapper):
+                    widget = wrapper.widget
+                else:
+                    widget = wrapper.wrappedWidget()
+
+                if widget is None:
+                    continue
+
+                value = wrapper.parameterValue()
+                parameters[param.name()] = value
+
+                if not param.checkValueIsAcceptable(value):
+                    raise AlgorithmDialogBase.InvalidParameterValue(param, widget)
             else:
+                if self.in_place and param.name() == 'OUTPUT':
+                    parameters[param.name()] = 'memory:'
+                    continue
+
                 dest_project = None
                 if not param.flags() & QgsProcessingParameterDefinition.FlagHidden and \
                         isinstance(param, (QgsProcessingParameterRasterDestination,
@@ -120,15 +162,15 @@ class AlgorithmDialog(QgsProcessingAlgorithmDialogBase):
 
         return self.algorithm().preprocessParameters(parameters)
 
-    def accept(self):
-        feedback = self.createFeedback()
-        context = dataobjects.createContext(feedback)
+    def runAlgorithm(self):
+        self.feedback = self.createFeedback()
+        self.context = dataobjects.createContext(self.feedback)
 
         checkCRS = ProcessingConfig.getSetting(ProcessingConfig.WARN_UNMATCHING_CRS)
         try:
             parameters = self.getParameterValues()
 
-            if checkCRS and not self.algorithm().validateInputCrs(parameters, context):
+            if checkCRS and not self.algorithm().validateInputCrs(parameters, self.context):
                 reply = QMessageBox.question(self, self.tr("Unmatching CRS's"),
                                              self.tr('Parameters do not all use the same CRS. This can '
                                                      'cause unexpected results.\nDo you want to '
@@ -137,7 +179,7 @@ class AlgorithmDialog(QgsProcessingAlgorithmDialogBase):
                                              QMessageBox.No)
                 if reply == QMessageBox.No:
                     return
-            ok, msg = self.algorithm().checkParameterValues(parameters, context)
+            ok, msg = self.algorithm().checkParameterValues(parameters, self.context)
             if not ok:
                 QMessageBox.warning(
                     self, self.tr('Unable to execute algorithm'), msg)
@@ -159,12 +201,12 @@ class AlgorithmDialog(QgsProcessingAlgorithmDialogBase):
             self.setInfo(
                 QCoreApplication.translate('AlgorithmDialog', '<b>Algorithm \'{0}\' starting&hellip;</b>').format(self.algorithm().displayName()), escapeHtml=False)
 
-            feedback.pushInfo(self.tr('Input parameters:'))
+            self.feedback.pushInfo(self.tr('Input parameters:'))
             display_params = []
             for k, v in parameters.items():
-                display_params.append("'" + k + "' : " + self.algorithm().parameterDefinition(k).valueAsPythonString(v, context))
-            feedback.pushCommandInfo('{ ' + ', '.join(display_params) + ' }')
-            feedback.pushInfo('')
+                display_params.append("'" + k + "' : " + self.algorithm().parameterDefinition(k).valueAsPythonString(v, self.context))
+            self.feedback.pushCommandInfo('{ ' + ', '.join(display_params) + ' }')
+            self.feedback.pushInfo('')
             start_time = time.time()
 
             if self.iterateParam:
@@ -176,29 +218,30 @@ class AlgorithmDialog(QgsProcessingAlgorithmDialogBase):
                     pass
 
                 self.cancelButton().setEnabled(self.algorithm().flags() & QgsProcessingAlgorithm.FlagCanCancel)
-                if executeIterating(self.algorithm(), parameters, self.iterateParam, context, feedback):
-                    feedback.pushInfo(
+                if executeIterating(self.algorithm(), parameters, self.iterateParam, self.context, self.feedback):
+                    self.feedback.pushInfo(
                         self.tr('Execution completed in {0:0.2f} seconds').format(time.time() - start_time))
                     self.cancelButton().setEnabled(False)
-                    self.finish(True, parameters, context, feedback)
+                    self.finish(True, parameters, self.context, self.feedback)
                 else:
                     self.cancelButton().setEnabled(False)
                     self.resetGui()
             else:
-                command = self.algorithm().asPythonCommand(parameters, context)
+                command = self.algorithm().asPythonCommand(parameters, self.context)
                 if command:
                     ProcessingLog.addToLog(command)
+                QgsGui.instance().processingRecentAlgorithmLog().push(self.algorithm().id())
                 self.cancelButton().setEnabled(self.algorithm().flags() & QgsProcessingAlgorithm.FlagCanCancel)
 
                 def on_complete(ok, results):
                     if ok:
-                        feedback.pushInfo(self.tr('Execution completed in {0:0.2f} seconds').format(time.time() - start_time))
-                        feedback.pushInfo(self.tr('Results:'))
-                        feedback.pushCommandInfo(pformat(results))
+                        self.feedback.pushInfo(self.tr('Execution completed in {0:0.2f} seconds').format(time.time() - start_time))
+                        self.feedback.pushInfo(self.tr('Results:'))
+                        self.feedback.pushCommandInfo(pformat(results))
                     else:
-                        feedback.reportError(
+                        self.feedback.reportError(
                             self.tr('Execution failed after {0:0.2f} seconds').format(time.time() - start_time))
-                    feedback.pushInfo('')
+                    self.feedback.pushInfo('')
 
                     if self.feedback_dialog is not None:
                         self.feedback_dialog.close()
@@ -207,19 +250,33 @@ class AlgorithmDialog(QgsProcessingAlgorithmDialogBase):
 
                     self.cancelButton().setEnabled(False)
 
-                    self.finish(ok, results, context, feedback)
+                    if not self.in_place:
+                        self.finish(ok, results, self.context, self.feedback)
+                    elif ok:
+                        self.close()
 
-                if not (self.algorithm().flags() & QgsProcessingAlgorithm.FlagNoThreading):
+                    self.feedback = None
+                    self.context = None
+
+                if not self.in_place and not (self.algorithm().flags() & QgsProcessingAlgorithm.FlagNoThreading):
                     # Make sure the Log tab is visible before executing the algorithm
                     self.showLog()
 
-                    task = QgsProcessingAlgRunnerTask(self.algorithm(), parameters, context, feedback)
+                    task = QgsProcessingAlgRunnerTask(self.algorithm(), parameters, self.context, self.feedback)
                     task.executed.connect(on_complete)
                     self.setCurrentTask(task)
                 else:
+                    self.proxy_progress = QgsProxyProgressTask(QCoreApplication.translate("AlgorithmDialog", "Executing “{}”").format(self.algorithm().displayName()))
+                    QgsApplication.taskManager().addTask(self.proxy_progress)
+                    self.feedback.progressChanged.connect(self.proxy_progress.setProxyProgress)
                     self.feedback_dialog = self.createProgressDialog()
                     self.feedback_dialog.show()
-                    ok, results = execute(self.algorithm(), parameters, context, feedback)
+                    if self.in_place:
+                        ok, results = execute_in_place(self.algorithm(), parameters, self.context, self.feedback)
+                    else:
+                        ok, results = execute(self.algorithm(), parameters, self.context, self.feedback)
+                    self.feedback.progressChanged.disconnect()
+                    self.proxy_progress.finalize(ok)
                     on_complete(ok, results)
 
         except AlgorithmDialogBase.InvalidParameterValue as e:
