@@ -23,7 +23,7 @@ __copyright__ = '(C) 2012, Victor Olaya'
 
 # This will get replaced with a git SHA1 when you do a git archive
 
-__revision__ = 'bc43194061452b37c8a22edcb7b380055a0c4236'
+__revision__ = 'f6ddc62fdb8ae589d3827e84d0c25bf669931040'
 
 import os
 import warnings
@@ -64,17 +64,17 @@ class MultipleInputDialog(BASE, WIDGET):
         self.selectedoptions = selectedoptions or []
 
         # Additional buttons
-        self.btnSelectAll = QPushButton(self.tr('Select all'))
+        self.btnSelectAll = QPushButton(self.tr('Select All'))
         self.buttonBox.addButton(self.btnSelectAll,
                                  QDialogButtonBox.ActionRole)
-        self.btnClearSelection = QPushButton(self.tr('Clear selection'))
+        self.btnClearSelection = QPushButton(self.tr('Clear Selection'))
         self.buttonBox.addButton(self.btnClearSelection,
                                  QDialogButtonBox.ActionRole)
-        self.btnToggleSelection = QPushButton(self.tr('Toggle selection'))
+        self.btnToggleSelection = QPushButton(self.tr('Toggle Selection'))
         self.buttonBox.addButton(self.btnToggleSelection,
                                  QDialogButtonBox.ActionRole)
         if self.datatype is not None:
-            btnAddFile = QPushButton(QCoreApplication.translate("MultipleInputDialog", 'Add file(s)…'))
+            btnAddFile = QPushButton(QCoreApplication.translate("MultipleInputDialog", 'Add File(s)…'))
             btnAddFile.clicked.connect(self.addFiles)
             self.buttonBox.addButton(btnAddFile,
                                      QDialogButtonBox.ActionRole)
