@@ -23,13 +23,16 @@ __copyright__ = '(C) 2012, Victor Olaya'
 
 # This will get replaced with a git SHA1 when you do a git archive
 
-__revision__ = '8a899c8758b30adc3f8d6df682a5702dd3bfe6f0'
+__revision__ = 'f6ddc62fdb8ae589d3827e84d0c25bf669931040'
 
 import os
-from qgis.core import QgsApplication, QgsProcessingModelAlgorithm
+from qgis.core import (QgsApplication,
+                       QgsProcessingModelAlgorithm,
+                       QgsProject)
 from qgis.PyQt.QtWidgets import QMessageBox
 from qgis.PyQt.QtCore import QCoreApplication
 from processing.gui.ContextAction import ContextAction
+from processing.modeler.ProjectProvider import PROJECT_PROVIDER_ID
 
 
 class DeleteModelAction(ContextAction):
@@ -41,12 +44,29 @@ class DeleteModelAction(ContextAction):
         return isinstance(self.itemData, QgsProcessingModelAlgorithm)
 
     def execute(self):
+        model = self.itemData
+        if model is None:
+            return # shouldn't happen, but let's be safe
+
+        project_provider = model.provider().id() == PROJECT_PROVIDER_ID
+
+        if project_provider:
+            msg = self.tr('Are you sure you want to delete this model from the current project?', 'DeleteModelAction')
+        else:
+            msg = self.tr('Are you sure you want to delete this model?', 'DeleteModelAction')
+
         reply = QMessageBox.question(
             None,
             self.tr('Delete Model', 'DeleteModelAction'),
-            self.tr('Are you sure you want to delete this model?', 'DeleteModelAction'),
+            msg,
             QMessageBox.Yes | QMessageBox.No,
             QMessageBox.No)
+
         if reply == QMessageBox.Yes:
-            os.remove(self.itemData.sourceFilePath())
-            QgsApplication.processingRegistry().providerById('model').refreshAlgorithms()
+            if project_provider:
+                provider = QgsApplication.processingRegistry().providerById(PROJECT_PROVIDER_ID)
+                provider.remove_model(model)
+                QgsProject.instance().setDirty(True)
+            else:
+                os.remove(model.sourceFilePath())
+                QgsApplication.processingRegistry().providerById('model').refreshAlgorithms()
